/*
* Copyright (c) 2017 STEALTHbits Technologies, Inc.
* All Rights Reserved
*/

#ifndef ACTIVITY_MONITOR_H
#define ACTIVITY_MONITOR_H

/*
* Implementation notes:
*
* Strings are encoded with UTF-8
*/

#include <stdint.h>
#include <time.h>

#ifdef __cplusplus
extern "C" {
#endif

	/*
	* AUDIT_API
	* API calling convention for this interface
	*/
#define ACTIVITY_API __stdcall

	/*
	* ACTIVITY_STATUS
	* A code that represents to the caller the success or failure of the request
	*/
	typedef unsigned int ACTIVITY_STATUS;

	typedef enum HostMonitorState_t {
		HostMonitorState_Disabled = 0,
		HostMonitorState_Enabled = 1
	} HostMonitorState;

#define ACTIVITY_OPERATION_FILE_READ             0x001
#define ACTIVITY_OPERATION_FILE_UPDATE           0x002
#define ACTIVITY_OPERATION_FILE_ADD              0x004
#define ACTIVITY_OPERATION_FILE_RENAME           0x008
#define ACTIVITY_OPERATION_FILE_DELETE           0x010
#define ACTIVITY_OPERATION_FILE_PERMISSION       0x020

#define ACTIVITY_OPERATION_DIRECTORY_ADD         0x040
#define ACTIVITY_OPERATION_DIRECTORY_RENAME      0x080
#define ACTIVITY_OPERATION_DIRECTORY_DELETE      0x100
#define ACTIVITY_OPERATION_DIRECTORY_PERMISSION  0x200

	/*
	* ActivityMonitorHostConfig
	*
	* represents the configuration options available for a host exposed
	* by the activity monitor
	*/

	typedef struct ActivityMonitorHostConfig_t {
		HostMonitorState state;
		/*
		* the bitwise combination of operations that are being
		* monitored for this host
		*/
		unsigned int operationsMask;

		/*
		* a list of accounts that are excluded from monitoring,
		* specified in the format of a security identifier
		*/
		char ** accountExclusions;
		unsigned int accountExclusionCount;

		/*
		* a list of paths that are included for monitoring,
		* this value is platform dependent
		*/
		char ** pathInclusions;
		unsigned int pathInclusionCount;

		/*
		* a list of paths that are excluded from monitoring,
		* this value is platform dependent
		*/
		char ** pathExclusions;
		unsigned int pathExclusionCount;

		/*
		* a free-form text description to associate with this host,
		* this could include user or environment related information
		*/
		char * comment;

		/*
		* the location of the syslog server that data is to be sent,
		* tcp://192.168.0.1:2000, tcp://localhost:2000,
		* protocols are one of udp:// or tcp://, tls://
		*/
		char * syslogUri;

		/*
		* vendor or diagnostic options that are not present in this
		* structure, for example on NetApp devices to set the fpolicy address
		* you can use the following: { "fpolicy-ip-address", "192.168.0.100" }
		*/
		char ** extraOptions;
		unsigned int extraOptionsCount;
	} ActivityMonitorHostConfig;

	typedef enum ActivityMonitorPlatform_t {
		ActivityMonitorPlatform_NetApp,
		ActivityMonitorPlatform_EmcCelerra,
		ActivityMonitorPlatform_EmcIsilon,
		ActivityMonitorPlatform_EmcUnity,
		ActivityMonitorPlatform_Hitachi,
		ActivityMonitorPlatform_SharePoint,
		ActivityMonitorPlatform_Nasuni
	} ActivityMonitorPlatform;

	typedef enum ActivityMonitorStatus_t {
		ActivityMonitorStatus_ok,
		ActivityMonitorStatus_warning,
		ActivityMonitorStatus_failure
	} ActivityMonitorStatus;

	/*
	* ActivityMonitorHost
	*
	* represents the full set of information about a monitored host
	*/

	typedef struct ActivityMonitorHost_t {
		/*
		* the name of the host that is being monitored
		*/
		char hostName[256];
		ActivityMonitorPlatform platform;
		char * message;
		ActivityMonitorStatus status;

		/*
		* the UTC timestamp of the last event received
		*/
		__time64_t eventReceivedTime;
		/*
		* pointer to configuration related information for this host
		*/
		ActivityMonitorHostConfig * config;
	} ActivityMonitorHost;

#define ACTIVITY_SUCCESS           0
#define ACTIVITY_INTERNAL_ERROR    1
#define ACTIVITY_INVALID_PARAMETER 2
#define ACTIVITY_HOST_NOT_FOUND    3

	/*
	* InitializeActivityMonitor --
	*
	* Requests the monitor initialize with a specific API version, if the DLL can support
	* this level of the API it will return successfully
	*/

	typedef void (ACTIVITY_API * LPACTIVITYMONITORLOGGER)(void *, const char *);

	typedef struct ActivityMonitorErrorChannel_t {
		LPACTIVITYMONITORLOGGER logger;
		void * userData;
	} ActivityMonitorErrorChannel;

#define ACTIVITY_API_VERSION 0x100

#ifdef ACTIVITY_MONITOR_PROTOTYPES
	ACTIVITY_STATUS ACTIVITY_API InitializeActivityMonitor(unsigned int apiVersion, ActivityMonitorErrorChannel * logger);
#endif /* ACTIVITY_MONITOR_PROTOTYPES */

	typedef ACTIVITY_STATUS(ACTIVITY_API * LPINITIALIZEACTIVITYMONITOR)(unsigned int apiVersion, ActivityMonitorErrorChannel * logger);

	/*
	* DestroyActivityMonitor --
	*
	* Release any globally allocated resources associated with this library
	*/

#ifdef ACTIVITY_MONITOR_PROTOTYPES
	ACTIVITY_STATUS ACTIVITY_API DestroyActivityMonitor();
#endif /* ACTIVITY_MONITOR_PROTOTYPES */

	typedef ACTIVITY_STATUS(ACTIVITY_API * LPDESTROYACTIVITYMONITOR)();

	/*
	* GetMonitoredHosts --
	*
	* Retrieve a set of hosts that are currently monitored, optionally specify in the hostName parameter
	* a specific host to be retrieved. Memory is allocated by the function and should not be modified.
	*/

	typedef struct ActivityMonitorHostSet_t {
		ActivityMonitorHost ** hosts;
		unsigned int count;
	} ActivityMonitorHostSet;

#ifdef ACTIVITY_MONITOR_PROTOTYPES
	ACTIVITY_STATUS ACTIVITY_API GetMonitoredHosts(const char * hostName, ActivityMonitorHostSet ** data);
#endif /* ACTIVITY_MONITOR_PROTOTYPES */

	typedef ACTIVITY_STATUS(ACTIVITY_API * LPGETMONITOREDHOSTS)(const char * hostName, ActivityMonitorHostSet_t ** data);

	/*
	* DestroyMonitoredHosts --
	*
	* Deallocates the memory associated with the list of hosts.
	*/

#ifdef ACTIVITY_MONITOR_PROTOTYPES
	ACTIVITY_STATUS ACTIVITY_API DestroyMonitoredHosts(ActivityMonitorHostSet_t * data);
#endif /* ACTIVITY_MONITOR_PROTOTYPES */

	typedef ACTIVITY_STATUS(ACTIVITY_API * LPDESTROYMONITOREDHOSTS)(ActivityMonitorHostSet_t * data);

	/*
	* SetMonitoredHost --
	*
	* Apply the configuration provided to the host name, this is intended only to
	* modify hosts that exist already
	*/

#ifdef ACTIVITY_MONITOR_PROTOTYPES
	ACTIVITY_STATUS ACTIVITY_API SetMonitoredHost(const char * hostName, ActivityMonitorHostConfig * config);
#endif /* ACTIVITY_MONITOR_PROTOTYPES */

	typedef ACTIVITY_STATUS(ACTIVITY_API * LPSETMONITOREDHOST)(const char * hostName, ActivityMonitorHostConfig * config);


	/*
	* SetActivityMonitorDebug --
	*
	* Debugging logging is disabled by default, call this API with a value of TRUE to turn this on
	*/

#ifdef ACTIVITY_MONITOR_PROTOTYPES
	ACTIVITY_STATUS ACTIVITY_API SetActivityMonitorDebug(int enable);
#endif /* ACTIVITY_MONITOR_PROTOTYPES */

	typedef ACTIVITY_STATUS(ACTIVITY_API * LPSETACTIVITYMONITORDEBUG)(int enable);

#ifdef __cplusplus
}
#endif
#endif /* ACTIVITY_MONITOR_H */